@@ -490,7 +490,7 @@ class ConsoleWidget(MetaQObjectHasTraits('NewBase', (LoggingConfigurable, superQ
         if self.paging == 'vsplit':
             height = height * 2 + splitwidth
 
-        return QtCore.QSize(width, height)
+        return QtCore.QSize(int(width), int(height))
 
     #---------------------------------------------------------------------------
     # 'ConsoleWidget' public interface
@@ -642,16 +642,7 @@ class ConsoleWidget(MetaQObjectHasTraits('NewBase', (LoggingConfigurable, superQ
             self._append_plain_text('\n')
             self._input_buffer_executing = self.input_buffer
             self._executing = True
-            self._prompt_finished()
-
-            # The maximum block count is only in effect during execution.
-            # This ensures that _prompt_pos does not become invalid due to
-            # text truncation.
-            self._control.document().setMaximumBlockCount(self.buffer_size)
-
-            # Setting a positive maximum block count will automatically
-            # disable the undo/redo history, but just to be safe:
-            self._control.setUndoRedoEnabled(False)
+            self._finalize_input_request()
 
             # Perform actual execution.
             self._execute(source, False)
@@ -677,6 +668,27 @@ class ConsoleWidget(MetaQObjectHasTraits('NewBase', (LoggingConfigurable, superQ
         """
         self._html_exporter.export()
 
+    def _finalize_input_request(self):
+        """
+        Set the widget to a non-reading state.
+        """
+        # Must set _reading to False before calling _prompt_finished
+        self._reading = False
+        self._prompt_finished()
+
+        # There is no prompt now, so before_prompt_position is eof
+        self._append_before_prompt_cursor.setPosition(
+            self._get_end_cursor().position())
+
+        # The maximum block count is only in effect during execution.
+        # This ensures that _prompt_pos does not become invalid due to
+        # text truncation.
+        self._control.document().setMaximumBlockCount(self.buffer_size)
+
+        # Setting a positive maximum block count will automatically
+        # disable the undo/redo history, but just to be safe:
+        self._control.setUndoRedoEnabled(False)
+
     def _get_input_buffer(self, force=False):
         """ The text that the user has entered entered at the current prompt.
 
@@ -944,7 +956,6 @@ class ConsoleWidget(MetaQObjectHasTraits('NewBase', (LoggingConfigurable, superQ
 
         # Perform the insertion.
         result = insert(cursor, input, *args, **kwargs)
-
         return result
 
     def _append_block(self, block_format=None, before_prompt=False):
@@ -1022,9 +1033,8 @@ class ConsoleWidget(MetaQObjectHasTraits('NewBase', (LoggingConfigurable, superQ
                 cursor.insertText(prefix)
                 current_pos = cursor.position()
 
-            cursor.movePosition(QtGui.QTextCursor.Left, n=len(prefix))
-            self._completion_widget.show_items(cursor, items)
-
+            self._completion_widget.show_items(cursor, items,
+                                               prefix_length=len(prefix))
 
     def _fill_temporary_buffer(self, cursor, text, html=False):
         """fill the area below the active editting zone with text"""
@@ -2020,12 +2030,11 @@ class ConsoleWidget(MetaQObjectHasTraits('NewBase', (LoggingConfigurable, superQ
         if buffer_size > 0:
             text = self._get_last_lines(text, buffer_size)
 
-        if self._executing:
-            viewport = self._control.viewport()
-            end_scroll_pos = self._control.cursorForPosition(
-                QtCore.QPoint(viewport.width()-1, viewport.height()-1)
-                ).position()
-            end_doc_pos = self._get_end_pos()
+        viewport = self._control.viewport()
+        end_scroll_pos = self._control.cursorForPosition(
+            QtCore.QPoint(viewport.width()-1, viewport.height()-1)
+            ).position()
+        end_doc_pos = self._get_end_pos()
 
         cursor.beginEditBlock()
         if self.ansi_codes:
@@ -2087,8 +2096,12 @@ class ConsoleWidget(MetaQObjectHasTraits('NewBase', (LoggingConfigurable, superQ
             cursor.insertText(text)
         cursor.endEditBlock()
 
-        if self._executing and end_doc_pos - end_scroll_pos <= 1:
-            self._control.moveCursor(QtGui.QTextCursor.End)
+        if end_doc_pos - end_scroll_pos <= 1:
+            end_scroll = (self._control.verticalScrollBar().maximum()
+                          - self._control.verticalScrollBar().pageStep())
+            # Only scroll down
+            if end_scroll > self._control.verticalScrollBar().value():
+                self._control.verticalScrollBar().setValue(end_scroll)
 
     def _insert_plain_text_into_buffer(self, cursor, text):
         """ Inserts text into the input buffer using the specified cursor (which
@@ -2402,13 +2415,13 @@ class ConsoleWidget(MetaQObjectHasTraits('NewBase', (LoggingConfigurable, superQ
             maximum = document.size().height()
             step = viewport_height
         diff = maximum - scrollbar.maximum()
-        scrollbar.setRange(0, maximum)
-        scrollbar.setPageStep(step)
+        scrollbar.setRange(0, round(maximum))
+        scrollbar.setPageStep(round(step))
 
         # Compensate for undesirable scrolling that occurs automatically due to
         # maximumBlockCount() text truncation.
         if diff < 0 and document.blockCount() == document.maximumBlockCount():
-            scrollbar.setValue(scrollbar.value() + diff)
+            scrollbar.setValue(round(scrollbar.value() + diff))
 
     def _custom_context_menu_requested(self, pos):
         """ Shows a context menu at the given QPoint (in widget coordinates).
